﻿Imports System.Security.Cryptography
Imports System.Text
Imports System.IO
Imports System.Environment
Namespace pasGenHelper
    ''' <summary>
    ''' manager-class for several functions pasGen needs
    ''' </summary>
    Public Class pasGenFunc
        Private m_appLang As String
        Private m_strEncrypted As String
        Private m_strDecrypted As String
        Private m_datPath As String = Environment.GetFolderPath(SpecialFolder.ApplicationData)
        Private m_dataFile As New FileInfo(m_datPath & "\pasGenData")
        ''' <summary>
        ''' ReadOnly Property for the encrypted Text
        ''' </summary>
        ''' <returns>
        ''' returns the encrypted Text
        ''' </returns>
        Public ReadOnly Property strEncrypted As String
            Get
                strEncrypted = m_strEncrypted
            End Get
        End Property
        ''' <summary>
        ''' Property for the decrypted Text
        ''' </summary>
        ''' <returns>
        ''' returns the decrypted Text
        ''' </returns>
        Public ReadOnly Property strDecrypted As String
            Get
                strDecrypted = m_strDecrypted
            End Get
        End Property
        ''' <summary>
        ''' Function to check the Password Strength
        ''' </summary>
        ''' <returns>
        ''' returns the strength of given Password
        ''' </returns>
        ''' <param name="pwLength" >
        ''' the length of the given Password
        ''' </param>
        ''' <param name="strPassword">
        ''' the Password to chek the Strength
        ''' </param>
        Public Function CheckPasswordStrength(ByVal pwLength As Integer, ByVal strPassword As String) As Integer
            Dim boolLowerCase As Boolean = False
            Dim boolUpperCase As Boolean = False
            Dim boolDigits As Boolean = False
            Dim boolSpecialChar As Boolean = False
            For Each chPW As Char In strPassword
                Select Case chPW
                    Case "a"c To "z"c
                        boolLowerCase = True
                    Case "A"c To "Z"c
                        boolUpperCase = True
                    Case "0"c To "9"c
                        boolDigits = True
                    Case Else
                        boolSpecialChar = True
                End Select
            Next
            Dim intStrength As Integer = 0
            If boolLowerCase Then intStrength += 1
            If boolUpperCase Then intStrength += 1
            If boolDigits Then intStrength += 1
            If boolSpecialChar Then intStrength += 1
            If pwLength >= 8 Then intStrength += 1
            Return intStrength
        End Function
        ''' <summary>
        ''' Function to crate a Password
        ''' </summary>
        ''' <returns>
        ''' returns the crated Password
        ''' </returns>
        ''' <param name="pwLength" >
        ''' the length of the Password to create
        ''' </param>
        ''' <param name="AllChar">
        ''' Optional: if no Chars are choosed, AllChar will be used to create a Password
        ''' </param>
        Public Function CreatePassword(pwLength As Integer, Optional AllChar As String = "ABCDEFGHJKLMNOPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz0123456789!@$?_-[](){}=§$%#&/") As String
            Dim chars As String = AllChar
            Dim rnd As New Random()
            Return New String(Enumerable.Repeat(chars, pwLength).[Select](Function(s) s(rnd.[Next](s.Length))).ToArray())
        End Function
        ''' <summary>
        ''' Sub to check if pasGen starts the first time
        ''' </summary>
        Public Sub checkStart()
            If m_dataFile.Exists Then
                My.Settings.firstStart = False
            Else
                My.Settings.firstStart = True
            End If
        End Sub
        ''' <summary>
        ''' Sub to Load an existing Database
        ''' </summary>
        Public Sub LoadDataSet(ByVal myDS As DataSet)
            myDS.Clear()
            If m_dataFile.Exists Then
                Using stream = New FileStream(m_dataFile.FullName, FileMode.Open), zipper = New Compression.GZipStream(stream, Compression.CompressionMode.Decompress)
                    myDS.ReadXml(zipper)
                End Using
            End If
        End Sub
        ''' <summary>
        ''' Sub to Save an existing Database
        ''' </summary>
        Public Sub SaveDataSet(ByVal myDS As DataSet)
            Using stream = New FileStream(m_dataFile.FullName, FileMode.Create), zipper = New Compression.GZipStream(stream, Compression.CompressionMode.Compress)
                myDS.WriteXml(zipper)
            End Using
            myDS.AcceptChanges()
        End Sub
        ''' <summary>
        ''' Function to crate a MD5 String Hash
        ''' </summary>
        ''' <returns>
        ''' returns the created Hash
        ''' </returns>
        ''' <param name="strString" >
        ''' the String to generate a Hash
        ''' </param>
        Public Function MD5StringHash(ByVal strString As String) As String
            Dim MD5 As New MD5CryptoServiceProvider
            Dim Data As Byte()
            Dim Result As Byte()
            Dim Res As String = ""
            Dim Tmp As String = ""
            Data = Encoding.ASCII.GetBytes(strString)
            Result = MD5.ComputeHash(Data)
            For i As Integer = 0 To Result.Length - 1
                Tmp = Hex(Result(i))
                If Len(Tmp) = 1 Then Tmp = "0" & Tmp
                Res += Tmp
            Next
            Return Res
        End Function
        ''' <summary>
        ''' Function to enrypt the given Password
        ''' </summary>
        ''' <returns>
        ''' returns the encrypted Password
        ''' </returns>
        ''' <param name="cryptKey" >
        ''' AES Key size (e.g. 128, 256)
        ''' </param>
        ''' <param name="enCrypted" >
        ''' the password to encrypt
        ''' </param>
        ''' <param name="cryptPW" >
        ''' the Masterpassword to encrypt Data
        ''' </param>
        Public Function pwEncryption(ByVal cryptKey As Int32, ByVal enCrypted As String, ByVal cryptPW As String) As String
            Dim crypSalt() As Byte
            crypSalt = Encoding.UTF8.GetBytes("eR2IYQ/BvOoV4Vc9hLfcb4xgO4SDCQJpBciZPcwOx3d+sW9w6oLNNTiQqrB+Uo/g3E5muWHO0CA/vvQiGil4tg==")
            Dim rfcKey As New Rfc2898DeriveBytes(cryptPW, crypSalt)
            Dim manAES As New AesManaged
            manAES.KeySize = cryptKey
            manAES.BlockSize = 128
            manAES.Key = rfcKey.GetBytes(manAES.KeySize \ 8)
            manAES.IV = rfcKey.GetBytes(manAES.BlockSize \ 8)
            Dim memStream As New MemoryStream
            Dim cryptStream As New CryptoStream(memStream, manAES.CreateEncryptor(),
              CryptoStreamMode.Write)
            Dim encodeData() As Byte
            encodeData = Encoding.UTF8.GetBytes(enCrypted)
            cryptStream.Write(encodeData, 0, encodeData.Length)
            cryptStream.FlushFinalBlock()
            cryptStream.Close()
            m_strEncrypted = Convert.ToBase64String(memStream.ToArray)
            memStream.Close()
            manAES.Clear()
            Return m_strEncrypted
        End Function
        ''' <summary>
        ''' Function to decrypt the given Password
        ''' </summary>
        ''' <returns>
        ''' returns the decrypted Password
        ''' </returns>
        ''' <param name="cryptKey" >
        ''' AES Key size (e.g. 128, 256)
        ''' </param>
        ''' <param name="deCrypted" >
        ''' the password to decrypt
        ''' </param>
        ''' <param name="cryptPW" >
        ''' the Masterpassword to decrypt Data
        ''' </param>
        Public Function pwDecryption(ByVal cryptKey As Int32, ByVal deCrypted As String, ByVal cryptPW As String) As String
            Dim crypSalt() As Byte
            crypSalt = Encoding.UTF8.GetBytes("eR2IYQ/BvOoV4Vc9hLfcb4xgO4SDCQJpBciZPcwOx3d+sW9w6oLNNTiQqrB+Uo/g3E5muWHO0CA/vvQiGil4tg==")
            Dim rfcKey As New Rfc2898DeriveBytes(cryptPW, crypSalt)
            Dim manAES As New AesManaged
            manAES.KeySize = cryptKey
            manAES.BlockSize = 128
            manAES.Key = rfcKey.GetBytes(manAES.KeySize \ 8)
            manAES.IV = rfcKey.GetBytes(manAES.BlockSize \ 8)
            Dim memStream As New IO.MemoryStream
            Dim cryptStream As New CryptoStream(memStream, manAES.CreateDecryptor(),
              CryptoStreamMode.Write)
            Try
                Dim encodeData() As Byte
                encodeData = Convert.FromBase64String(deCrypted)
                cryptStream.Write(encodeData, 0, encodeData.Length)
                cryptStream.FlushFinalBlock()
                cryptStream.Close()
                m_strDecrypted = Encoding.UTF8.GetString(memStream.ToArray)
                memStream.Close()
                manAES.Clear()
            Catch ex As Exception
                m_strDecrypted = "Not to decrypt! - wrong Password?"
            End Try
            Return m_strDecrypted
        End Function
        ''' <summary>
        ''' Sub to switch the language
        ''' </summary>
        Public Sub SetLanguage()
            m_appLang = My.Settings.appLanguage
            Dim Culture As Globalization.CultureInfo
            Try
                Culture = New Globalization.CultureInfo(m_appLang)
                If Culture Is Nothing Then
                    m_appLang = "en-US"
                    Culture = New Globalization.CultureInfo(m_appLang)
                End If
            Catch ex As Exception
                MessageBox.Show(My.Resources.msgLangID, My.Resources.msgTitle, MessageBoxButtons.OK, MessageBoxIcon.Error)
                m_appLang = "en-US"
                Culture = New Globalization.CultureInfo(m_appLang)
            End Try
            System.Threading.Thread.CurrentThread.CurrentUICulture = Culture
        End Sub
        Public Async Sub delClipBoard()
            Await Task.Delay(15000)
            Clipboard.Clear()
        End Sub
    End Class
End Namespace

